/****************************************************************************
 *
 * Copyright (c) 2005 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/
/*
 * Follow the bouncing comments in this file to see how Log4c can be used
 * within Hula.
 * 
 * You may need to set LD_LIBRARY_PATH=/path/to/hula/lib/ to run this example.
 * 
 * The Log4c library looks for logging configuration files in:
 * ${LOG4C_RCPATH}/log4crc
 * ${HOME}/.log4crc
 * ./log4crc
 * The example config file for this program has two categories defined,
 * "root" which is the parent for everything (all events bubble to it),
 * and "funtest", a category representing events from this code.
 * "root" is set to output to syslog, but only LOG_FATAL severity messages.
 * "funtest" outputs to stderr, any severity.
 * I would leave these notes in the config file, but Log4c's XML parser
 * seems to choke on comments, and uses no other config file formats.
 */

#define LOGGERNAME "funtest"
/*
 * recommended:
 *   define LOGGERNAME in your source file to categorize logging output
 *   alternately, specify a category during each logging call (see below)
 * default:
 *   LOGGERNAME will be "root" if you do not define it
 */

#include <stdio.h>
#include <log4c.h>
/*
 * required:
 *   include log4c.h for necessary logging functions
 */

int main(int argc, char** argv)
{
    int i;

    LogStartup();
    /*
     * required:
     *   call LogStartup() before making any logging calls
     */

    printf("logging some one-liner events...\n");
    Log(LOG_WARN, "this is a quick warning event");
    /*
     * Log(level, message) implies LOGGERNAME as the category
     */

    LogMsg("root", LOG_FATAL, "this is a quick event to root");
    /*
     * LogMsg(category, level, message) will use an expressed category
     */
    Log(LOG_ALERT, "brought to you by the number %d...", i);
    LogMsg("root", LOG_FATAL, "...and the letter %s", "H");
    /*
     * Log and LogMsg will accept printf-style formatting and arguments
     */

    printf("spinning through categories...\n");
    for(i=0; i<LOG4C_PRIORITY_UNKNOWN; i++) {
	Log(i*100, "this is a %s event", log4c_priority_to_string(i*100));
	/*
	 * log4c provides a bunch of other functions beyond the hula-wrapped
	 * Log and LogMsg, if those are not detailed enough.  I won't get
	 * into those here though.
	 */
    }

    printf("quitting\n");

    LogShutdown();
    /*
     * required:
     *   call LogShutdown() when done logging
     * beware:
     *   if Log4c is outputting to stdout or stderr, it will be closed
     */

    return 0;
}
