__inline static long
CheckForNMAPCommError(long result)
{
    if ((result > 999) && (result < 10000)) {
        return(result);
    }
    
    return(STATUS_NMAP_COMM_ERROR);
}

__inline static long
SendError(Connection *conn, char *prefix, char *command, long errorValue)
{
    long ccode;
    
    if (errorValue != STATUS_ABORT) {
        if ((errorValue > 0) && (errorValue < STATUS_MAX)) {
            if (Imap.command.returnValueIndex[errorValue] != -1) {
                ccode = ConnWriteF(conn, ImapErrorStrings[Imap.command.returnValueIndex[errorValue]].formatString, prefix, command);
            } else {
                ccode = ConnWriteF(conn, "%s NO %s undefined internal error %d\r\n", prefix, command, errorValue);
            }
        } else if ((errorValue < 10000) || (errorValue > 999)) {
            ccode = ConnWriteF(conn, "%s NO %s message store returned error %d\r\n", prefix, command, errorValue);
        } else {
            ccode = ConnWriteF(conn, "%s NO %s undefined internal error %d\r\n", prefix, command, errorValue);
        }

        if (ccode != -1) {
            return(errorValue);
        }
    }

    return(STATUS_ABORT);
}

__inline static long
SendOk(ImapClient *client, unsigned char *Command)
{
    if (ConnWriteF(client->conn, "%s OK %s completed\r\n", client->Ident, Command) != -1) {
        return(STATUS_CONTINUE);
    }
    return(STATUS_ABORT);
}

__inline static long 
GetMessageUidRange(ImapClient *client, char *range, unsigned long *rangeStart, unsigned long *rangeEnd)
{
    char *colonPtr;
    unsigned long start;
    unsigned long end;
    unsigned long messageCount;
    long ccode;

    messageCount = client->Messages;
    /* the mailbox is not empty */
    if ((colonPtr = strchr(range, ':')) != NULL) {
        /* parse the set */
        start = (unsigned long)atol(range);     
        if (start > 0) {
            ;
        } else if (*range == '*') {
            start = UID_HIGHEST;
        } else {
            return(STATUS_INVALID_ARGUMENT);
        }

        if (*(colonPtr + 1) == '*') {
            end = UID_HIGHEST;
        } else {
            end = (unsigned long)atol(colonPtr + 1);
            if (end != 0) {
                ;
            } else {
                return(STATUS_INVALID_ARGUMENT);
            }
        }

        /* switch if the larger number is first */
        if (start <= end) {
            ;
        } else {
            unsigned long tmp;

            tmp = start;
            start = end;
            end = tmp;
        }

        return(UIDtoNUMRange(client, start, end, rangeStart, rangeEnd));
    }

    /* Just a single number */
    if (*range == '*') {
        /* Special: Last available item */
        *rangeStart = messageCount - 1;
        *rangeEnd = *rangeStart;
        return(STATUS_CONTINUE);
    }

    *rangeStart = (unsigned long)atol(range);

    ccode = UIDtoNUM(client, rangeStart);
    if (ccode == STATUS_CONTINUE) {
        *rangeEnd = *rangeStart;
        return(STATUS_CONTINUE);
    }

    return(STATUS_UID_NOT_FOUND);  /* nothing found */
}

__inline static long 
GetMessageIdRange(ImapClient *client, char *range, unsigned long *rangeStart, unsigned long *rangeEnd)
{
    char *colonPtr;
    unsigned long start;
    unsigned long end;
    unsigned long messageCount;

    messageCount = client->Messages;
    if (messageCount > 0) {
        /* the mailbox is not empty */
        if ((colonPtr = strchr(range, ':')) != NULL) {
            /* We've got a set ! */
            start = (unsigned long)atol(range);     
            if (start > 0) {
                ;
            } else if (*range == '*') {
                start = messageCount;
            } else {
                return(STATUS_INVALID_ARGUMENT);
            }

            if (*(colonPtr + 1) == '*') {
                end = messageCount;
            } else {
                end = (unsigned long)atol(colonPtr + 1);
                if (end != 0) {
                    ;
                } else {
                    return(STATUS_INVALID_ARGUMENT);
                }
            }

            /* switch if the larger number is first */
            if (start <= end) {
                ;
            } else {
                unsigned long tmp;

                tmp = start;
                start = end;
                end = tmp;
            }

            start--;
            end--;

            if (end < messageCount) {
                *rangeStart = start;
                *rangeEnd = end;
                return(STATUS_CONTINUE);
            }
            return(STATUS_INVALID_ARGUMENT);
        }

        /* Just a single number */
        if (*range == '*') {
            /* Special: Last available item */
            *rangeStart = messageCount - 1;
            *rangeEnd = *rangeStart;
            return(STATUS_CONTINUE);
        }

        start = (unsigned long)atol(range);

        start--;
        if (start < messageCount) {
            *rangeStart = start;
            *rangeEnd = *rangeStart;
            return(STATUS_CONTINUE);
        }
    }

    return(STATUS_INVALID_ARGUMENT);
}


__inline static long 
GetMessageRange(ImapClient *client, char **nextRange, unsigned long *rangeStart, unsigned long *rangeEnd, BOOL byUid)
{
    char *currentRange;
    char *commaPtr;
    long ccode;

    currentRange = *nextRange;

    if ((commaPtr = strchr(currentRange, ',')) == NULL) {
        if (byUid) {
            ccode = GetMessageUidRange(client, currentRange, rangeStart, rangeEnd);   
        } else {
            ccode = GetMessageIdRange(client, currentRange, rangeStart, rangeEnd);
        }
        *nextRange = NULL;
        return(ccode);
    }

    *commaPtr = '\0';
    if (byUid) {
        ccode = GetMessageUidRange(client, currentRange, rangeStart, rangeEnd);   
    } else {
        ccode = GetMessageIdRange(client, currentRange, rangeStart, rangeEnd);
    }
    *commaPtr = ',';
    *nextRange = commaPtr + 1;
    return(ccode);
}

__inline static long
NMAPtoNUM(ImapClient *client, unsigned long nmapId, unsigned long *num)
{
    register unsigned long start;
    register unsigned long end;
    register unsigned long mean;
    long delta;

    if (client->Messages > 0) {
        start = 0;
        end = client->Messages;
        mean = end >> 1;

        do {
            if (client->IDList[mean] > nmapId) {
                end = mean;
            } else {
                start = mean;
            }
            delta = end - start;
            mean = start + (delta >> 1);
        } while (delta > 1);

        if (nmapId == client->IDList[start]) {
            *num = start + 1;
            return(STATUS_CONTINUE);
        }
    }

    return(STATUS_NMAPID_NOT_FOUND);

}

#define UNSEEN_FLAGS_BLOCK_SIZE 16

__inline static void
RememberFlags(unsigned long *unseenFlags, unsigned long msgNum, unsigned long msgFlags)
{
    if (unseenFlags ==  NULL) {
        unseenFlags = MemMalloc((1 + UNSEEN_FLAGS_BLOCK_SIZE) * 2 * sizeof(unsigned long));
        if (unseenFlags) {
            unseenFlags[2] = msgNum;
            unseenFlags[3] = msgFlags;
            /* Set Counters */
            unseenFlags[0] = UNSEEN_FLAGS_BLOCK_SIZE;       /* Allocated */
            unseenFlags[1] = 1;                             /* Used */
        }
    } else {
        if (unseenFlags[1] < unseenFlags[0]) {
            unseenFlags[2 + (unseenFlags[1] * 2)]   = msgNum;
            unseenFlags[2 + (unseenFlags[1] * 2) + 1]  = msgFlags;
            /* Set Counter */
            unseenFlags[1]++;
        } else {
            unsigned long *Tmp;
            Tmp = MemRealloc(unseenFlags, ((1 + unseenFlags[0] + UNSEEN_FLAGS_BLOCK_SIZE) * 2 * sizeof(unsigned long)));
            if (Tmp) {
                unseenFlags = Tmp;
                unseenFlags[2 + (unseenFlags[1] * 2)]  = msgNum;
                unseenFlags[2 + (unseenFlags[1] * 2) + 1] = msgFlags;
                /* Set Counters */
                unseenFlags[0] += UNSEEN_FLAGS_BLOCK_SIZE;
                unseenFlags[1]++;
            }
        }
    }
}


__inline static void
FreeUnseenFlags(ImapClient *client)
{
    if (client->UnseenFlags) {
        MemFree(client->UnseenFlags);
        client->UnseenFlags = NULL;
    }
}

__inline static long
SendUnseenFlags(ImapClient *client)
{
    long ccode;
    unsigned long i;
    char *space = "";
    unsigned long MsgFlags;
    unsigned long imapId;

    if (!client->UnseenFlags) {
        return(STATUS_CONTINUE);
    }

    for (i = 0; i < client->UnseenFlags[1]; i++) {
        MsgFlags = client->UnseenFlags[2 + (i * 2) + 1];
        
        ccode = NMAPtoNUM(client, client->UnseenFlags[2 + (i * 2)], &imapId);
        if (ccode == STATUS_CONTINUE) {
            ccode = ConnWriteF(client->conn, "* %lu FETCH (FLAGS (", imapId);
            if ((MsgFlags & MSG_STATE_ANSWERED) && (ccode != -1)) {
                ccode = ConnWrite(client->conn, "\\Answered", sizeof("\\Answered") - 1);
                space = " ";
            }

            if ((MsgFlags & MSG_STATE_DELETED) && (ccode != -1)) {
                ccode = ConnWriteF(client->conn, "%s\\Deleted", space);
                space = " ";
            }

            if ((MsgFlags & MSG_STATE_RECENT) && (ccode != -1)) {
                ccode = ConnWriteF(client->conn, "%s\\Recent", space);
                space = " ";
            }

            if ((MsgFlags & MSG_STATE_READ) && (ccode != -1)) {
                ccode = ConnWriteF(client->conn, "%s\\Seen", space);
                space = " ";
            }

            if ((MsgFlags & MSG_STATE_DRAFT) && (ccode != -1)) {
                ccode = ConnWriteF(client->conn, "%s\\Draft", space);
                space = " ";
            }

            if ((MsgFlags & MSG_STATE_PRIOHIGH) && (ccode != -1)) {
                ccode = ConnWriteF(client->conn, "%s\\Flagged", space);
                space = " ";
            }

            if (ccode != -1) {
                ccode = ConnWrite(client->conn, "))\r\n", sizeof("))\r\n") - 1);
            }

            if (ccode != -1) {
                continue;
            }
            return(STATUS_ABORT);
        }

        break;
    }

    MemFree(client->UnseenFlags);
    client->UnseenFlags = NULL;
    return(STATUS_CONTINUE);
}


__inline static void
CheckPathLen(char *path)
{
    if (strlen(path) >= XPL_MAX_PATH) {
        path[XPL_MAX_PATH - 1] = '\0';
    }
}

__inline static void
RemoveMboxSpaces(char *mbox)
{
    unsigned char *ptr = mbox;

    while((ptr = strchr(ptr, ' ')) != NULL) {
        *ptr++ = 127;
    }
}

__inline static void
AddMboxSpaces(char *mbox)
{
    unsigned char *ptr = mbox;

    while((ptr = strchr(ptr, 127)) != NULL) {
        *ptr++ = ' ';
    }
}
