/****************************************************************************
 * <Novell-copyright>
 * Copyright (c) 2001 Novell, Inc. All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, contact Novell, Inc.
 * 
 * To contact Novell about this file by physical or electronic mail, you 
 * may find current contact information at www.novell.com.
 * </Novell-copyright>
 ****************************************************************************/
#include <config.h>

#include "hula-mgrp.h"

#include <mwtempl.h>

#define PRODUCT_NAME              "Hula Manager"
#define PRODUCT_SHORT_NAME        "Hula"
#define PRODUCT_DESCRIPTION       ""

static int DMCReadAnswer(
        Connection *conn,
        unsigned char *response,
        size_t length,
        BOOL checkForResult);

HulaManagerConfig                  HulaManager;

/*  fixme - the Exiting boolean is misued!
*/
BOOL                              Exiting;

struct {
/*    RegistrationStates state; */
    unsigned char access[DMC_HASH_SIZE];
    MDBHandle directoryHandle;
    SSL_CTX *context;
    BOOL debug;
} DMCLibrary = {
/*    REGISTRATION_LOADING, */
    { '\0' },
    NULL,
    NULL,
    FALSE
};
    
static BOOL 
HulaManagerCheckConfiguration(void)
{
    int i;
    BOOL result = FALSE;
    MDBValueStruct *config = NULL;

    config = MDBCreateValueStruct(HulaManager.directory.handle, NULL);
    if (config) {
        HulaManager.directory.tree[0] = '\\';

        result = MDBGetServerInfo(
                HulaManager.directory.host,
                HulaManager.directory.tree + 1,
                config);
    } else {
        XplConsolePrintf("Hula directory driver not responding; shutting down.\r\n");
    }
    if (result) {
        i = MDBReadDN(
                HulaManager.directory.host,
                (unsigned char *) MSGSRV_A_HULA_MESSAGING_SERVER,
                config);
        if (i) {
            strcpy(
                    (char *) HulaManager.directory.dn,
                    (char *) config->Value[0]);
        } else {
            i = MDBReadDN(
                    HulaManager.directory.host,
                    (unsigned char *) MSGSRV_A_MESSAGING_SERVER,
                    config);
            if (i) {
                strcpy(
                        (char *) HulaManager.directory.dn,
                        (char *) config->Value[0]);

                i = MDBWriteDN(
                        HulaManager.directory.host,
                        (unsigned char *) MSGSRV_A_HULA_MESSAGING_SERVER,
                        config);
                if (i) {
                    MDBClear(
                            HulaManager.directory.host,
                            (unsigned char *) MSGSRV_A_MESSAGING_SERVER,
                            config);
                } else {
                    result = FALSE;
                    XplConsolePrintf("Hula messaging server not upgraded; shutting down.\r\n");
                }
            } else {
                result = FALSE;
                XplConsolePrintf("Hula messaging server not configured; shutting down.\r\n");
            }
        }
        MDBFreeValues(config);
        if (result) {
            HulaManager.directory.rdn =
                (unsigned char *) strrchr(
                        (char *) HulaManager.directory.dn,
                        '\\') + 1;
            i = MDBRead(
                    HulaManager.directory.dn,
                    (unsigned char *) MSGSRV_A_SERVER_STATUS,
                    config);
            if (i) {
                HulaManager.flags |= HULAMRG_FLAG_FAILED_SHUTDOWN;
            }
        }
        MDBFreeValues(config);
    } else {
        XplConsolePrintf("Hula directory host not configured; shutting down.\r\n");
    }
    if (config) {
        MDBDestroyValueStruct(config);
        config = NULL;
    }
    return(result);
}

static BOOL 
HulaManagerLoadConfiguration(void)
{
    BOOL result = FALSE;
    MDBValueStruct *list = NULL;
    MDBValueStruct *config = NULL;
    HulaManagerAgent *agent;

    config = MDBCreateValueStruct(HulaManager.directory.handle, NULL);
    agent = MemMalloc(sizeof(HulaManagerAgent));
    if (agent) {
        agent->flags = HULAMRG_FLAG_MODULE_ENABLED | HULAMRG_FLAG_MODULE_DMC;
        agent->data = NULL;
        agent->load = 0;
        strcpy((char *) agent->name, MSGSRV_NLM_DMC);
        result = HulaManagerAgentPrep(agent);
        HulaManager.pDMC = agent;
    } else {
        result = FALSE;
    }
    if (config) {
        MDBDestroyValueStruct(config);
        config = NULL;
    }
    if (list) {
        MDBDestroyValueStruct(list);
        list = NULL;
    }
    return(result);
}

static void 
HulaManagerUnloadConfiguration(void)
{
    HulaManagerAgentRelease(HulaManager.pDMC);
    return;
}

void 
HulaManagerStartDMC(void)
{
    if ((HulaManager.pDMC->flags & HULAMRG_FLAG_MODULE_ENABLED) &&
            !(HulaManager.pDMC->flags & HULAMRG_FLAG_MODULE_LOADED)) {
        if (HulaManagerLoadAgent(HulaManager.pDMC) == TRUE) {
            HulaManager.pDMC->flags |= HULAMRG_FLAG_MODULE_LOADED;
        }
    }
    return;
}

void 
HulaManagerStopAgents(void)
{
    if (HulaManager.pDMC->flags & HULAMRG_FLAG_MODULE_LOADED) {
        HulaManagerUnloadAgent(HulaManager.pDMC);
    }
    return;
}

void 
HulaManagerUnload(void)
{
    HulaManagerUnloadConfiguration();
    if (HulaManager.directory.handle) {
        MsgShutdown();
    }
    CONN_TRACE_SHUTDOWN();
    ConnShutdown();
    if (HulaManager.logging) {
        LoggerClose(HulaManager.logging);
        HulaManager.logging = NULL;
    }
    /*
    ** Cleanup SSL
    */
    ERR_free_strings();
    ERR_remove_state(0);
    EVP_cleanup();

    XPLCryptoLockDestroy();

    IPCleanup();

    return;
}

BOOL 
HulaManagerLoad(HulaManagerFlags Flags)
{
    unsigned long i;
    unsigned long err;
    unsigned char buffer[256];
    BOOL result;
    MD5_CTX context;

    HulaManager.directory.handle = NULL;

    HulaManager.flags = Flags;
    HulaManager.state = HULAMRG_STATE_INITIALIZING;

    HulaManager.pDMC = NULL;

    HulaManager.times.reload = 5 * 60;

    HulaManager.signo = SIGTERM;

    result = MemoryManagerOpen((unsigned char *) PRODUCT_SHORT_NAME);
    if (result) {
        IPInit();

        XplGetHighResolutionTime(i);

        SSL_load_error_strings();
        SSL_library_init();

        XPLCryptoLockInit();

        srand((unsigned int)i);

        MD5_Init(&context);

        for (i = 0; i < sizeof(buffer); ) {
            buffer[i] = (unsigned char)(rand());

            if (buffer[i]) {
                i++;
            }
        }

        buffer[sizeof(buffer) - 1] = '\0';

        MD5_Update(&context, buffer, sizeof(buffer) - 1);

        MD5_Final(buffer, &context);

        RAND_seed(buffer, 16);

    	HulaManager.logging = LoggerOpen("hulamanager");
        if (HulaManager.logging == NULL) {
            err = 0;
            XplConsolePrintf("Hula manager failed to acquire a logging handle; error %lu.\r\n", err);
        }

        result = ConnStartup(15 * 60, FALSE);
    }

    if (result) {
        result = MDBInit();
    } else {
        XplConsolePrintf("Hula connection management library failed to initialize.\r\n");
    }

    if (result) {
        HulaManager.directory.handle = (MDBHandle *)MsgInit();
        if (HulaManager.directory.handle) {
            result = HulaManagerCheckConfiguration();
            if (result) {
                result = DMCInitialize(HulaManager.directory.handle);
            }
        } else {
            XplConsolePrintf("Hula messaging library failed to initialize.\r\n");
        }
    } else {
        XplConsolePrintf("Hula directory interface library failed to initialize.\r\n");
    }

    if (result) {
        CONN_TRACE_INIT((char *)MsgGetWorkDir(NULL), "manager");
        // CONN_TRACE_SET_FLAGS(CONN_TRACE_ALL); /* uncomment this line and pass '--enable-conntrace' to autogen to get the agent to trace all connections */

        result = HulaManagerLoadConfiguration();
    } else {
        XplConsolePrintf("Host not configured for Hula\r\n");
    }

    if (result) {
        HulaManager.state = HULAMRG_STATE_RUNNING;
    } else {
        HulaManager.state = HULAMRG_STATE_UNLOADING;
    }

    return(result);
}

BOOL 
DMCInitialize(MDBHandle directoryHandle)
{
    BOOL result = FALSE;
    MDBValueStruct *v;

    if (directoryHandle) {
        DMCLibrary.directoryHandle = directoryHandle;

        v = MDBCreateValueStruct(DMCLibrary.directoryHandle, NULL);
        if (v) {
            MDBRead(
                    (unsigned char *) MSGSRV_ROOT,
                    (unsigned char *) MSGSRV_A_ACL,
                    v);
            if (v->Used) {
                result = HashCredential(
                        MsgGetServerDN(NULL),
                        v->Value[0],
                        DMCLibrary.access);
            }
            MDBDestroyValueStruct(v);
        }
    }
    return(result);
}

BOOL 
DMCAuthenticate(
        Connection *pConn,
        unsigned char *response,
        int length)
{
    int ccode = -1;

    ccode = DMCReadAnswer(pConn, response, length, TRUE);
    switch (ccode) {
        case 1000: {
                int                i;
                unsigned char    *ptr;
                unsigned char    *salt;
                unsigned char    digest[16];
                unsigned char    message[33];
                MD5_CTX        context;

                ptr = (unsigned char *) strchr((char *) response, '\"');
                if (ptr) {
                    salt = ++ptr;
                    if ((ptr = (unsigned char *) strchr((char *) ptr, '\"')) != NULL) {
                        *ptr = '\0';
                    }
                    MD5_Init(&context);
                    MD5_Update(&context, salt, strlen((char *) salt));
                    MD5_Update(&context, DMCLibrary.access, DMC_HASH_SIZE);
                    MD5_Final(digest, &context);
                    DMCEncrypt(pConn, response, length, FALSE);
                    ConnWrite(pConn, "AUTH ", 5);
                    for (i = 0; i < 16; i++) {
                        sprintf((char *) message + (i * 2), "%02x", digest[i]);
                    }
                    ConnWrite(pConn, (char *) message, 32);
                    ConnWrite(pConn, "\r\n", 2);
                    if ((i = ConnFlush(pConn)) == 39) {
                        if ((i = DMCReadAnswer(
                                        pConn,
                                        response,
                                        length,
                                        TRUE)) == 1000) {
                            return(TRUE);
                        }
                    }
                }
                /*
                **  Fall through to the default case statement.
                */
            }
        default: {
                break;
            }
    }
    return(FALSE);
}

BOOL 
DMCEncrypt(Connection *pConn, unsigned char *response, int length, BOOL force)
{
    int ccode;
    BOOL result;

    result = pConn->ssl.enable = FALSE;
    if (!force && XplIsLocalIPAddress(pConn->socketAddress.sin_addr.s_addr)) {
        return(TRUE);
    }
    if (DMCLibrary.context) {
        setsockopt(
                pConn->socket,
                IPPROTO_TCP,
                1,
                (unsigned char *)&ccode,
                sizeof(ccode));
        if (((ccode = ConnWrite(pConn, "CAPA\r\n", 6)) != -1) &&
                ((ccode = ConnFlush(pConn)) != -1)) {
            while ((ccode = DMCReadAnswer(
                            pConn,
                            response,
                            length,
                            TRUE)) == 2001) {
                if (strcmp((char *) response, "TLS Encryption Extension") != 0) {
                    continue;
                }
                result = TRUE;
            }
            if (ccode != 1000) {
                result = FALSE;
            }
        }
        if (result &&
                ((pConn->ssl.context = SSL_new(DMCLibrary.context)) != NULL) &&
                ((ccode = ConnWrite(pConn, "TLS\r\n", 5)) != -1) &&
                ((ccode = ConnFlush(pConn)) != -1) &&
                ((ccode = DMCReadAnswer(pConn, response, length, TRUE)) == 1000) &&
                ((ccode = SSL_set_bsdfd(pConn->ssl.context, pConn->socket)) == 1) &&
                ((ccode = SSL_connect(pConn->ssl.context)) == 1)) {
            pConn->ssl.enable = TRUE;
            return(TRUE);
        }
        if (pConn->ssl.context) {
            SSL_free(pConn->ssl.context);
            pConn->ssl.context = NULL;
        }
    }
    return(FALSE);
}


static int
DMCReadAnswer(
        Connection *pConn,
        unsigned char *response,
        size_t length,
        BOOL checkForResult)
{
    int len;
    int result = -1;
    size_t count;
    unsigned char *cur;

    len = ConnReadAnswer(pConn, (char *) response, length);
    if (len > 0) {
        cur = response + 4;
        if (checkForResult &&
                ((*cur == ' ') ||
                 (*cur == '-') ||
                 ((cur = (unsigned char * ) strchr((char *) response, ' ')) != NULL))) {
            *cur++ = '\0';
            result = atoi((char *) response);
            count = len - (cur - response);
            memmove(response, cur, count);
            response[count] = '\0';
        } else {
            result = atoi((char *) response);
        }
    }
    return(result);
}

void 
DMCQuit(Connection *pConn)
{
    ConnWrite(pConn, "QUIT\r\n", 6);
    CONN_TCP_CLOSE(pConn);
    return;
}

BOOL
DMCStart(Connection *pConn)
{
    int ccode = -1;
    unsigned char response[CONN_BUFSIZE + 1];

    ConnWrite(pConn, "START", 5);
    ConnWrite(pConn, "\r\n", 2);
    ConnFlush(pConn);
    if ((ccode = DMCReadAnswer(
                    pConn,
                    response,
                    CONN_BUFSIZE,
                    TRUE)) == 1000) {
        return(TRUE);
    }
    return(FALSE);
}

BOOL
DMCShutdown(Connection *pConn)
{
    int ccode = -1;
    unsigned char response[CONN_BUFSIZE + 1];

    ConnWrite(pConn, "SHUTDOWN", 8);
    ConnWrite(pConn, "\r\n", 2);
    ConnFlush(pConn);
    if ((ccode = DMCReadAnswer(
                    pConn,
                    response,
                    CONN_BUFSIZE,
                    TRUE)) == 1000) {
        return(TRUE);
    }
    return(FALSE);
}

