/*
 *  (c) Copyright 2005 Novell, Inc.
 *  All Rights Reserved.
 *
 * 
 *  File:    hulamdbtest.c
 *  Purpose: MDB Test & Demo App
 *  Project: Hula
 *
 *  $Revision:   1.0  $
 *  $Modtime:   28 Nov 2005 21:58:04  $
 *  $Author:   Betty Graupe  $
 *
 */


#include <stdio.h>
/*#include <platform.h> */
#include <config.h>
#include <xpl.h>

#include <mdb.h>
/* #include "hulamdbtest.h" */

#define F   FALSE
#define T   TRUE


#define MAX_VAL_FUNCS  5
typedef BOOL (*funcPtr)(void *);

/*  The testing array.  Just add new entries as you need them.               */
typedef struct _MDBTest {
    char *testMsg;                   /*  Msg to be displayed before test.    */
    funcPtr testFunc;                /*  Test function to be executed.       */
    char *arg1;                      /*  1st arg - generally obj DN          */
    char *arg2;                      /*  2nd arg - generally obj DN          */
    char *arg3;                      /*  2nd arg - generally obj DN          */
    BOOL bool1;
    BOOL bool2;
    BOOL bool3;
    BOOL bool4;
    BOOL bool5;
    funcPtr valFunc[MAX_VAL_FUNCS];  /*  Validation functions - up to MAX_...*/
    char *failMsg;                   /*  Set by called function if failure.  */
} MDBTest;

/*  The testing functions.                                                   */
BOOL initialize(void *testInfo);
BOOL getServerInfo(void *testInfo);
char treeName[MDB_MAX_TREE_CHARS];
char serverDN[MDB_MAX_OBJECT_CHARS];
BOOL getAPIVersion(void *testInfo);
char apiVersionDescription[256];
BOOL createObject(void *testInfo);
BOOL addAttribute(void *testInfo);

/*  The validation functions.                                                */
BOOL validate1(void *testInfo);

char result[1024];

MDBTest mdbTest[] =
{
    {"MDBInit(void)", &initialize, NULL,
            NULL, NULL, F,F,F,F,F, 
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {"MDBGetServerInfo(NULL, NULL, NULL)", &getServerInfo, NULL,
            NULL, NULL, F,F,F,F,F,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {"MDBGetServerInfo(serverDN, NULL, NULL)", &getServerInfo, serverDN,
            NULL, NULL, F,F,F,F,F,
            {&validate1, NULL, NULL, NULL, NULL},
            NULL},
    {"MDBGetServerInfo(serverDN, treeName, NULL)", &getServerInfo, serverDN,
            treeName, NULL, F,F,F,F,F,
            {&validate1, NULL, NULL, NULL, NULL},
            NULL},
    {"MDBGetAPIVersion(TRUE, NULL)", &getAPIVersion, apiVersionDescription,
            NULL, NULL, T,0,0,0,0,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {"MDBGetAPIVersion(FALSE, NULL)", &getAPIVersion, apiVersionDescription,
            NULL, NULL, F,0,0,0,0,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {"Create Object", &createObject, "\\tree\\context\\object",
            NULL, NULL, F,F,F,F,F,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {"Add Attribute", &addAttribute, "\\tree\\context\\object",
            NULL, NULL, F,F,F,F,F,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
    {NULL, NULL, NULL,
            NULL, NULL, 0,0,0,0,0,
            {NULL, NULL, NULL, NULL, NULL},
            NULL},
};

int
main(int argc, char *argv[])
{
    int i, j;
    int numTestSuccesses = 0, numTestFailures = 0;
    int numValSuccesses  = 0, numValFailures  = 0;

    printf ("\nMDB Tests Starting...\n");
    printf ("--------------------------------------------------------------\n");
    for (i = 0 ; mdbTest[i].testFunc != NULL ; i++) {
        printf ("Test %d: %s...\n", i+1, mdbTest[i].testMsg);

        /*  Only if the main test is successful, we run the validations.     */
        if ((mdbTest[i].testFunc)(&mdbTest[i])) {
            numTestSuccesses++;
            printf ("\tResult: Success.\n");
            for (j = 0 ; j < MAX_VAL_FUNCS ; j++) {
                if (mdbTest[i].valFunc[j] != NULL) {
                    if ((mdbTest[i].valFunc[j])(&mdbTest[i])) {
                        numValSuccesses++;
                        printf ("\tValidation succeeded.\n");
                    } else {
                        numValFailures++;
                        printf ("\tValidation failed.\n");
                    }
                }
            }
        } else {
            numTestFailures++;
            printf ("\tFailed!\n");
            if (mdbTest[i].failMsg != NULL) {
                printf ("\tResult: %s\n", mdbTest[i].failMsg);
                free(mdbTest[i].failMsg);
            }
        }
    }
    printf ("--------------------------------------------------------------\n");
    printf ("MDB Tests Complete.\n");
    printf ("Successful tests: %d\n", numTestSuccesses);
    printf ("Failed tests: %d\n", numTestFailures);
    printf ("Successful validations: %d\n", numValSuccesses);
    printf ("Failed validations: %d\n", numValFailures);
    printf ("--------------------------------------------------------------\n");

    return(0);

}  /*  End  main() */

BOOL
initialize(void *testInfo)
{
    if (!MDBInit()) {
        /*  This is the only function where we actually exit on failure.     */
        printf("Could not initialize MDB! Exiting tests...\n");
        exit(0);
    } else {
        return(TRUE);
    }

}  /*  End  initialize()  */


BOOL
getServerInfo(void *testInfo)
{
    MDBTest *info = (MDBTest *)testInfo;

    if (MDBGetServerInfo(info->arg1, info->arg2, NULL)) {
        if (info->arg1) {
            printf("\tserverDN = %s\n", info->arg1);
        }
        if (info->arg2) {
            printf("\ttreeName = %s\n", info->arg2);
        }
        return (TRUE);
    } else {
        info->failMsg = strdup("Could not retrieve server information.");
        return (FALSE);
    }

}  /*  End  getServerInfo()  */


BOOL
getAPIVersion(void *testInfo)
{
    MDBTest *info = (MDBTest *)testInfo;
    int     version = 0;

    version = MDBGetAPIVersion(info->bool1, info->arg1, NULL);
    if (version < 1) {
        info->failMsg = strdup("Could not retrieve valid MDB API version.");
        return (FALSE);
    }

    if (info->bool1) {
        printf("\tMDB implementation is compatible with MDB version %d.\n", version);
    } else {
        printf("\tMDB version = %d.\n", version);
    }

    if (info->arg1) {
        printf("\tapiDescription = %s\n", info->arg1);
    }

    return (TRUE);

}  /*  End  getAPIVersion()  */


BOOL
createObject(void *testInfo)
{
    return (TRUE);

}  /*  End  createObject()  */


BOOL
addAttribute(void *testInfo)
{
    return (TRUE);

}  /*  End  addAttribute()  */


BOOL
validate1(void *testInfo)
{
    return (TRUE);

}  /*  End  validate1()  */
