using System;
using System.Xml;
using System.IO;
using System.Collections;
using Hula.ICal;
using Hula.Util;

namespace Hula.Cal {

//<summary>
// ICalendar filter.  Allows subsetting as specified in the CalDav spep
//</summary>
public class CalFilter : Filter {
    class ComponentFilter {
 	public bool allComp;
	public bool allProp;
	public Hashtable components = null;
	public Hashtable properties = null;

	public ComponentFilter(XmlNode node) {
	    foreach (XmlNode child in node) {
		string name = null;
		switch (child.LocalName) {
		case "comp" :
		    name = child.Attributes["name"].Value;
		    if (components == null) {
			components = new Hashtable();
		    }
		    components[name] = new ComponentFilter(child);
		    break;
		case "prop" :
		    name = child.Attributes["name"].Value;
		    if (properties == null) {
			properties = new Hashtable();
		    }
		    properties[name] = true;
		    break;
		case "allprop" :
		    allProp = true;
		    break;
		case "allcomp" :
		    allComp = true;
		    break;
		}
	    }
	}
    }

    private ComponentFilter topFilter = null;

    public CalFilter() {
	topFilter = null;
    }

    public CalFilter(XmlNode node) {
	XmlNode child = node.FirstChild;
	if (child != null) {
	    topFilter = new ComponentFilter(child);
	}
    }

    public override void Run(TextReader reader, TextWriter writer) {
	ICalComponent obj = new ICalComponent(reader);

	StartDocument(writer);
	FormatComponent(writer, obj, topFilter);
	EndDocument(writer);
    }

    protected virtual void StartDocument(TextWriter writer) {
    }

    protected virtual void EndDocument(TextWriter writer) {
    }

    protected virtual void StartComponent(TextWriter writer, ICalComponent component) {
	writer.WriteLine("BEGIN:{0}", component.ComponentKind);
    }

    protected virtual void WriteProperty(TextWriter writer, ICalComponent component, ICalProperty property) {
	writer.Write(property.ToString());
    }

    protected virtual void EndComponent(TextWriter writer, ICalComponent component) {
	writer.WriteLine("END:{0}", component.ComponentKind);
    }

    private void FormatComponent(TextWriter writer, ICalComponent component, ComponentFilter filter) {
	StartComponent(writer, component);

	ICalProperty property = component.GetFirstProperty();
	while (property != null) {
	    if (filter == null
		|| filter.allProp
		|| (filter.properties != null
		    && filter.properties.Contains(property.Name))) {
		WriteProperty(writer, component, property);
	    }
	    property = component.GetNextProperty();
	}

	ICalComponent child = component.GetFirstComponent(ICalComponent.Kind.Any);
	while (child != null) {
	    if (filter == null 	     
		|| filter.allComp) {
		FormatComponent(writer, child, null);
	    } else if (filter.components != null
		       && filter.components.Contains(child.ComponentKind)) {
		FormatComponent(writer, child, (ComponentFilter)filter.components[child.ComponentKind]);
	    }

	    child = component.GetNextComponent(ICalComponent.Kind.Any);
	}
	EndComponent(writer, component);
    }
}
}
