using System;
using System.Xml;
using System.IO;
using System.Collections;
using Hula.ICal;
using Hula.Util;

namespace Hula.Cal {

//<summary>
// xCal -> iCal
//</summary>
public class XmlFilterCal : Filter {
    public XmlFilterCal() {
    }

    private string MaybeWriteElement (TextWriter writer, string pname,
				      Hashtable attrs, bool begin,
				      Stack scopes) {
	if (pname == "") {
	    return "";
	}

	if (begin) {
	    writer.Write("BEGIN:");
	    writer.WriteLine(pname);
	
	    foreach (string attr in attrs.Keys) {
		writer.Write(attr);
		writer.Write(":");
		writer.WriteLine(attrs[attr]);
	    }
	} else {
	    writer.Write(pname);
	    foreach (string attr in attrs.Keys) {
		writer.Write(";");
		writer.Write(attr);
		writer.Write("=");
		writer.Write(attrs[attr]);
	    }
	    writer.Write(":");
	}
	scopes.Push (begin);
	attrs.Clear ();
	return "";
    }

    public override void Run(TextReader reader, TextWriter writer) {
	XmlTextReader xml = new XmlTextReader(reader);

	string pname  = "";
	Hashtable attrs = new Hashtable ();
	Stack scopes = new Stack ();

	while (xml.Read()) {
	    if (xml.LocalName == "iCalendar") {
		continue;
	    }

	    switch(xml.NodeType) {
	    case XmlNodeType.Element :
		pname = MaybeWriteElement (writer, pname, attrs, true, scopes);

		/* buffer this until we know if it should be a BEGIN:
		 * or not */
		pname = xml.LocalName.ToUpper ();

		if (xml.MoveToFirstAttribute()) {
		    do {
			attrs[xml.LocalName.ToUpper()] = xml.Value;
		    } while (xml.MoveToNextAttribute());
		}
		break;
	    case XmlNodeType.EndElement :
		pname = MaybeWriteElement (writer, pname, attrs, false, scopes);
		if ((bool)scopes.Pop ()) {
		    writer.WriteLine("END:" + xml.LocalName.ToUpper());
		} else {
		    writer.WriteLine();
		}
		break;
	    case XmlNodeType.Text :
		pname = MaybeWriteElement (writer, pname, attrs, false, scopes);
		writer.Write(xml.Value);
		break;
	    }
	}
    }
}
}
