using System;
using System.Xml;
using System.Web;
using System.Text;
using System.Collections;
using System.IO;
using Lucene.Net.Search;
using Lucene.Net.Index;
using Lucene.Net.Documents;
using Hula.Store;
using Hula.Util;
using Hula.Cal;
using Hula.ICal;

namespace Hula.Dav {
public class AddressbookQueryReport : QueryReport {
    public AddressbookQueryReport(HttpContext context) : base (context) {
    }

    protected override void SetupQuery() {
	base.SetupQuery();
	query.Add(new TermQuery(new Term("hula.mime-type", "text/calendar")), true, false);
    }

    protected override void HandleNode(XmlNode node) {
	switch (node.LocalName) {
	case "filter" :
	    ParseFilter(node);
	    break;
	default :
	    base.HandleNode(node);
	    break;
	}
    }
       
    private void ParseComponentParamFilter (XmlNode node, string prop) {
	string name = node.Attributes["name"].Value;
	
	foreach (XmlNode child in node) {
	    switch (child.LocalName) {
	    case "is-defined" :		
		break;
	    case "text-match" :
		// FIXME: Don't do caseless param filters for now.
		TermQuery q = new TermQuery (new Term("vcard." + prop + ".param", + name + "=" + child.InnerText));

		query.Add(q, true, false);
		break;
	    }
	}
    }

    private void ParsePropFilter (XmlNode node) {
	string name = node.Attributes["name"].Value.ToLower();
	
	foreach (XmlNode child in node) {
	    switch (child.LocalName) {
	    case "is-defined" :		
		break;
	    case "text-match" :
		XmlAttribute caselessAttr = child.Attributes["caseless"];
		XmlAttribute matchTypeAttr = child.Attributes["match-type"];

		string prop;
		string value;

		if (caselessAttr != null && caselessAttr.Value == "yes") {
		    prop = "vcard." + name + ".value.lower";
		    value = child.InnerText.ToLower();
		} else {
		    prop = "vcard." + name + ".value";
		    value = child.InnerText;
		}

		string matchType;
		if (matchTypeAttr == null) {
		    matchType = "equals";
		} else {
		    matchType = matchTypeAttr.Value;
		}
		
		Query q;

		switch (matchType) {
  		case "equals" :
		    q = new TermQuery(new Term(prop, value));
		    break;
		case "contains" :
		    q = new SubstringQuery(new Term(prop, value));
		    break;
		case "starts-with" :
		    q = new PrefixQuery(new Term(prop, value));
		    break;
		case "ends-with" :
		    q = new WildcardQuery(new Term(prop, "*" + value));
		    break;
		default :
		    q = null;
		    break;
		}

		if (q != null) {
		    query.Add(q, true, false);
		}
		break;
	    case "param-filter" :
		ParseComponentParamFilter(child, name);
		break;
	    }
	}	
    }

    private void ParseFilter (XmlNode node) {
	foreach (XmlNode child in node) {
	    switch(child.LocalName) {
	    case "prop-filter" :
		ParsePropFilter(child);
		break;
	    }
	}
    }
}
}
