using System;
using System.IO;
using System.Runtime.InteropServices;
using Lucene.Net.Documents;

namespace Hula.ICal {

[StructLayout(LayoutKind.Sequential)]
public struct ICalTime {
    public int year;
    public int month;
    public int day;
    public int hour;
    public int minute;
    public int second;
    public int isUtc;
    public int isDate;
    public int isDaylight;
    
    public IntPtr zone;       /**< timezone */

    [DllImport(ICalComponent.icalLib)]
    private static extern ICalTime icaltime_convert_to_zone(ICalTime time,
							    IntPtr zone);
    [DllImport(ICalComponent.icalLib)]
    private static extern IntPtr icaltimezone_get_utc_timezone();
    [DllImport(ICalComponent.icalLib)]
    private static extern ICalTime icaltime_from_string(string str);
    [DllImport(ICalComponent.icalLib)]
    private static extern ICalTime icaltime_add(ICalTime time, ICalDuration duration);
    
    public ICalTime ToUniversalTime() {
	return icaltime_convert_to_zone(this, icaltimezone_get_utc_timezone());
    }

    public static ICalTime Parse(string str) {
	return icaltime_from_string(str);
    }

    public string ToString(DateTools.Resolution resolution) {
	if (resolution == DateTools.Resolution.YEAR) {
	    return String.Format("{0:d4}", year);
	} else if (resolution == DateTools.Resolution.MONTH) {
	    return String.Format("{0:d4}{1:d2}", year, month);
	} else if (resolution == DateTools.Resolution.DAY) {
	    return String.Format("{0:d4}{1:d2}{2:d2}", year, month, day);
	} else if (resolution == DateTools.Resolution.HOUR) {
	    return String.Format("{0:d4}{1:d2}{2:d2}{3:d2}", year, month, day, hour);
	} else if (resolution == DateTools.Resolution.MINUTE) {
	    return String.Format("{0:d4}{1:d2}{2:d2}{3:d2}{4:d2}", year, month, day, hour, minute);
	} else if (resolution == DateTools.Resolution.SECOND) {
	    return String.Format("{0:d4}{1:d2}{2:d2}{3:d2}{4:d2}", year, month, day, hour, minute);
	} else if (resolution == DateTools.Resolution.MILLISECOND) {
	    return String.Format("{0:d4}{1:d2}{2:d2}{3:d2}{4:d2}000", year, month, day, hour, minute);
	} else {
	    throw new System.ArgumentException("unknown resolution " + resolution);
	}
    }    

    public bool IsNull {
	get {
	    return (second + minute + hour + day + month + year == 0);
	}
    }

    public static ICalTime operator+ (ICalTime t, ICalDuration d) {
	return icaltime_add(t, d);
    }
    public static ICalTime operator+ (ICalDuration d, ICalTime t) {
	return icaltime_add(t, d);
    }

};
[StructLayout(LayoutKind.Sequential)]
public struct ICalDuration {
    [MarshalAs(UnmanagedType.SysInt)] public int is_neg;
    [MarshalAs(UnmanagedType.SysUInt)] public uint days;
    [MarshalAs(UnmanagedType.SysUInt)] public uint weeks;
    [MarshalAs(UnmanagedType.SysUInt)] public uint hours;
    [MarshalAs(UnmanagedType.SysUInt)] public uint minutes;
    [MarshalAs(UnmanagedType.SysUInt)] public uint seconds;
};

}
