using System;
using System.IO;
using System.Text;
using System.Collections;
using Lucene.Net;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Store;
using Hula.ICal;
using Hula.Store;

namespace Hula.Indexing {
public class CalIndexer : Indexer {
    private static Hashtable indexFields = new Hashtable();
    private static Hashtable everythingScores = new Hashtable();

    static CalIndexer() {
	indexFields["SUMMARY"] = true;
	indexFields["ORGANIZER"] = true;
	indexFields["ATTENDEE"] = true;
	indexFields["DESCRIPTION"] = true;
	indexFields["CATEGORIES"] = true;
	indexFields["LOCATION"] = true;

        everythingScores.Add("SUMMARY", (float)2.0);
        everythingScores.Add("ORGANIZER", null);
        everythingScores.Add("ATTENDEE", null);
        everythingScores.Add("DESCRIPTION", null);
        everythingScores.Add("CATEGORIES", null);
        everythingScores.Add("LOCATION", null);
    }

    public CalIndexer() {
    }

    public override void IndexDocument(LuceneIndex idx,
				       IStore store,
				       string guid) {
	TextReader reader;
	IDictionary properties;
	GetDocument(store, guid, out reader, out properties);
	using(reader) {
	    using (ICalComponent icalObject = new ICalComponent(reader)) {
		IndexDocument(idx, store, guid, properties, icalObject);
	    }
	}
    }

    private void IndexDocument(LuceneIndex idx, IStore store, string guid, IDictionary properties, ICalComponent obj) {
	ICalComponent comp = obj.GetFirstComponent(ICalComponent.Kind.Any);
	while (comp != null) {
	    IndexComponent(idx, store, guid, properties, obj, comp, comp.StartTime, comp.EndTime);
	    
	    int i = 0;
	    foreach (ICalTime time in comp.Recurrences) {
		IndexComponent(idx, store, guid, properties, obj, comp, time, time + comp.Duration);
		if (i++ > 1000) {
		    break;
		}
	    }

	    comp = obj.GetNextComponent(ICalComponent.Kind.Any);
	}
    }    

    private void IndexComponent(LuceneIndex idx, 
                                IStore store,
				string guid, 
				IDictionary properties, 
				ICalComponent obj, 
				ICalComponent comp,
				ICalTime start,
				ICalTime end) {
	Document doc = new Document();

        IDictionary contentFields = new Hashtable();

	ICalProperty property = obj.GetFirstProperty();
	while (property != null) {
	    string parentKey = "ical.parent." + property.Name.ToLower();
	    doc.Add(Field.Keyword(parentKey, property.Value));
	    property = obj.GetNextProperty();
	}

	doc.Add(Field.Keyword("ical.type", comp.ComponentKind));
	
	property = comp.GetFirstProperty();
	while (property != null) {
	    string valField = "ical." + property.Name.ToLower() + ".value";
	    doc.Add(new Field(valField, property.Value, Field.Store.NO, Field.Index.UN_TOKENIZED));
	    doc.Add(new Field(valField + ".lower", property.Value.ToLower(), Field.Store.NO, Field.Index.UN_TOKENIZED));

	    string paramField = "ical." + property.Name.ToLower() + ".param";
	    ICalParameter param = property.GetFirstParameter();
	    while (param != null) {
		doc.Add(new Field(paramField, param.ToString(), Field.Store.NO, Field.Index.UN_TOKENIZED));
		doc.Add(new Field(paramField, param.ToString(), Field.Store.NO, Field.Index.UN_TOKENIZED));
		param = property.GetNextParameter();
	    }

	    doc.Add(new Field("ical." + property.Name.ToLower(), property.ToString(), Field.Store.YES, Field.Index.NO));

	    if(indexFields[property.Name] != null) {
                contentFields[property.Name] = property.Value;
	    }
	    property = comp.GetNextProperty();
	}

	doc.Add(new Field("hula.start-time", start.ToString(DateTools.Resolution.MINUTE),
			  Field.Store.YES, Field.Index.UN_TOKENIZED));
	doc.Add(new Field("hula.end-time", end.ToString(DateTools.Resolution.MINUTE),
			  Field.Store.YES, Field.Index.UN_TOKENIZED));

	AddProperties(doc, properties);
	AddStandardFields(doc, store, guid, properties);
        AddContentFields(doc, contentFields, everythingScores);

	idx.Writer.AddDocument(doc);
    }
}
}
