/****************************************************************************
 *
 * Copyright (c) 2001-2002 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/
using System;
using System.Runtime.InteropServices;

namespace Hula.Mdb {
    public class MemMgr : IDisposable {
		private const string MEMMGR_LIB = "hulamemmgr";
		private readonly string DEFAULT_AGENT_NAME = "MDB# Agent";

		private string agentName;
		
		public string AgentName {
			get {
				return agentName;
			}
		}
		
		public MemMgr(string agentName) {
			this.agentName = agentName; 
			Init(agentName);
		}
		
		public MemMgr() {
			this.agentName = DEFAULT_AGENT_NAME;
			Init(agentName);
		}
		
		~MemMgr() {
			Cleanup();
		}
		
		public void Dispose() {
			Cleanup();
			System.GC.SuppressFinalize(this);
		}
		
		private void Init(string agentName) {
			MemoryManagerOpen(agentName);
		}
		
		private void Cleanup() {
			MemoryManagerClose(agentName);
		}
		
		[DllImport(MEMMGR_LIB)]
		public static extern bool MemoryManagerOpen(string agentName);
	
		[DllImport(MEMMGR_LIB)]
		public static extern bool MemoryManagerClose(string agentName);
	}
}
