/****************************************************************************
 *
 * Copyright (c) 2005 Novell, Inc.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, contact Novell, Inc.
 *
 * To contact Novell about this file by physical or electronic mail,
 * you may find current contact information at www.novell.com
 *
 ****************************************************************************/

using System;
using System.IO;
using log4net;

namespace Hula.Nmap
{
public enum DeliveryNotificationFlags {
    None = 0,
    Success = (1 << 0),
    Timeout = (1 << 1),
    Failure = (1 << 2),
    Header = (1 << 3),
    Body = (1 << 4)
}

public class AuthCommands : NmapCommands
{

    public AuthCommands(NmapConnection nconn) : base(nconn)
    {
    }

    public string CLR(string user)
    {
        nconn.WriteRequest("CLR {0}", user);
        NmapResponse response = nconn.ReadFullReply ();
        return response.text;
    }

    public void USER(string user)
    {
        nconn.WriteRequest("USER {0}", user);
        NmapResponse response = nconn.ReadFullReply ();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QCREA() {
	nconn.WriteRequest ("QCREA");
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QABRT() {
	nconn.WriteRequest ("QABRT");
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QRUN() {
	nconn.WriteRequest ("QRUN");
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_FROM(string sender, string authenticatedAddress) {
	nconn.WriteRequest ("QSTOR FROM {0} {1}", sender, authenticatedAddress);
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_LOCAL(string recipient, string originalAddress, DeliveryNotificationFlags flags) {
	nconn.WriteRequest ("QSTOR LOCAL {0} {1} {2}", recipient, originalAddress, (int)flags);
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_TO(string recipient, string originalAddress, DeliveryNotificationFlags flags) {
	nconn.WriteRequest ("QSTOR TO {0} {1} {2}", recipient, originalAddress, (int)flags);
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_CAL(string recipient, string originalAddress, DeliveryNotificationFlags flags) {
	nconn.WriteRequest ("QSTOR CAL {0} {1} {2}", recipient, originalAddress, (int)flags);
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_RAW(string line) {
	nconn.WriteRequest ("QSTOR RAW {0}, line");
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }

    public void QSTOR_MESSAGE(TextReader reader) {
	// FIXME: use a period-escaping filter of some sort
	string val = reader.ReadToEnd ();
	nconn.WriteRequest ("QSTOR MESSAGE {0}", val.Length);
	nconn.Writer.Write (val);
	nconn.Writer.Flush ();
	NmapResponse response = nconn.ReadFullReply();
	if (response.code != 1000) {
	    throw new NmapException(response);
	}
    }
}
}
