using Lucene.Net;
using Lucene.Net.Analysis.Standard;
using Lucene.Net.Documents;
using Lucene.Net.Index;
using Lucene.Net.Search;
using Lucene.Net.Store;
using NUnit.Framework;

namespace Hula.Util {

[TestFixture]
public class TestSubstringQuery {
    private RAMDirectory dir;
    private IndexWriter writer;
    private IndexSearcher searcher;

    [SetUp]
    public void SetUp() {
	dir = new RAMDirectory();
	writer = new IndexWriter(dir, new StandardAnalyzer(), true);

	Document doc = new Document();
	doc.Add(Field.Keyword("field1", "this is the first value in document 1"));
	doc.Add(Field.Keyword("field2", "this is the second value in document 1"));
	writer.AddDocument(doc);

	doc = new Document();
	doc.Add(Field.Keyword("field1", "this is the first value in document 2"));
	doc.Add(Field.Keyword("field2", "this is the SECOND VALUE in document 2"));
	writer.AddDocument(doc);

	doc = new Document();
	doc.Add(Field.Keyword("field1", "this is the first value in document 3"));
	doc.Add(Field.Keyword("field2", "Document 3 does not have the same value for the second field"));
	writer.AddDocument(doc);

	doc = new Document();
	doc.Add(Field.Keyword("field1", "this is the first value in document 4"));
	doc.Add(Field.Keyword("field3", "this is the third value in document 4"));

	writer.AddDocument(doc);		   

	writer.Close();

	searcher = new IndexSearcher(dir);
    }

    private Hits GetHits(string field, string substring, bool caseSensitive) {
	Term t = new Term(field, substring);
	SubstringQuery q = new SubstringQuery(t);
	q.IsCaseSensitive = caseSensitive;
	return searcher.Search(q);
    }  

    [Test]
    public void TestSubstrings() {	
	Hits hits = GetHits("field1", "first value", false);	    
	Assert.AreEqual(4, hits.Length());

	hits = GetHits("field1", "", false);	    
	Assert.AreEqual(4, hits.Length());

	hits = GetHits("field2", "second value", false);
	Assert.AreEqual(2, hits.Length());

	hits = GetHits("field2", "second value", true);
	Assert.AreEqual(1, hits.Length());

	hits = GetHits("field2", "first value", true);
	Assert.AreEqual(0, hits.Length());

	hits = GetHits("field3", "third value", true);
	Assert.AreEqual(1, hits.Length());
    }
}
}
